"use strict";

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }
function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _iterableToArrayLimit(r, l) { var t = null == r ? null : "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"]; if (null != t) { var e, n, i, u, a = [], f = !0, o = !1; try { if (i = (t = t.call(r)).next, 0 === l) { if (Object(t) !== t) return; f = !1; } else for (; !(f = (e = i.call(t)).done) && (a.push(e.value), a.length !== l); f = !0); } catch (r) { o = !0, n = r; } finally { try { if (!f && null != t["return"] && (u = t["return"](), Object(u) !== u)) return; } finally { if (o) throw n; } } return a; } }
function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }
function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }
function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }
function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
var fs = require('fs');
var path = require('path');
function capitalize(str) {
  return str[0].toUpperCase() + str.slice(1);
}
function camelCase(str) {
  return str.split('-').map(function (s, index) {
    return (index === 0 ? s[0].toLowerCase() : s[0].toUpperCase()) + s.slice(1).toLowerCase();
  }).join('');
}
var checkValidEntry = function checkValidEntry(_ref) {
  var include = _ref.include,
    exclude = _ref.exclude,
    entry = _ref.entry;
  var shouldInclude = !include || include.test(entry);
  var shouldExclude = exclude && exclude.test(entry);
  return shouldInclude && !shouldExclude;
};
var scanIcons = function scanIcons(dir, options) {
  var include = options.include,
    exclude = options.exclude,
    match = options.match;
  function readFilesRecursively(dir) {
    try {
      var files = [];
      var entries = fs.readdirSync(dir, {
        withFileTypes: true
      });
      entries.forEach(function (entry) {
        var entryPath = path.join(dir, entry.name);
        if (entry.isDirectory()) {
          files.push.apply(files, _toConsumableArray(readFilesRecursively(entryPath)));
        } else {
          var isValidEntry = checkValidEntry({
            include: include,
            exclude: exclude,
            entry: entryPath
          });
          if (isValidEntry && match) {
            var code = fs.readFileSync(entryPath, {
              encoding: 'utf-8'
            });
            var matches = code.match(match) || [];
            files.push.apply(files, _toConsumableArray(matches));
          }
        }
      });
      return files;
    } catch (error) {
      console.error("Dynamically icon loading error: ".concat(error));
      return [];
    }
  }
  return readFilesRecursively(dir);
};
var styles = {
  fas: 'solid',
  fat: 'thin',
  fal: 'light',
  fad: 'duotone',
  far: 'regular'
};
var generateIconImportExpression = function generateIconImportExpression(_ref2, style) {
  var iconList = _ref2.iconList,
    license = _ref2.license;
  var importIcons = iconList.map(function (icon) {
    var _icon$match = icon.match(/(\w-?)+/gi),
      _icon$match2 = _slicedToArray(_icon$match, 2),
      prefix = _icon$match2[0],
      name = _icon$match2[1];
    var iconStyle = styles[prefix];
    if (iconStyle !== style) return;
    var iconName = "fa".concat(capitalize(camelCase(name)));
    var dynamicIconName = "dynamic".concat(capitalize(iconName)).concat(capitalize(iconStyle));
    console.log({
      dynamicIconName: dynamicIconName
    });
    return "".concat(iconName, " as ").concat(dynamicIconName);
  }).filter(Boolean);
  importIcons = _toConsumableArray(new Set(importIcons)).join(', ');
  if (!importIcons.length) return '';
  return "import { ".concat(importIcons, " } from '@fortawesome/").concat(license, "-").concat(style, "-svg-icons';");
};
var generateDynamicIconList = function generateDynamicIconList(iconList) {
  var dynamicIcons = iconList.map(function (icon) {
    var _icon$match3 = icon.match(/(\w-?)+/gi),
      _icon$match4 = _slicedToArray(_icon$match3, 2),
      prefix = _icon$match4[0],
      name = _icon$match4[1];
    var iconName = "fa".concat(capitalize(camelCase(name)));
    var iconStyle = styles[prefix];
    return "dynamic".concat(capitalize(iconName)).concat(capitalize(iconStyle));
  });
  return _toConsumableArray(new Set(dynamicIcons)).join(', ');
};
var generateModuleScript = function generateModuleScript(_ref3) {
  var iconList = _ref3.iconList,
    _ref3$license = _ref3.license,
    license = _ref3$license === void 0 ? 'free' : _ref3$license;
  var generateIconExpressionByStyle = generateIconImportExpression.bind(null, {
    iconList: iconList,
    license: license
  });
  var dynamicIconList = generateDynamicIconList(iconList);
  return "\n\t\t\timport { library as iconLibrary } from '@fortawesome/fontawesome-svg-core';\n\t\t\t".concat(generateIconExpressionByStyle(styles.far), "\n\t\t\t").concat(generateIconExpressionByStyle(styles.fad), "\n\t\t\t").concat(generateIconExpressionByStyle(styles.fal), "\n\t\t\t").concat(generateIconExpressionByStyle(styles.fat), "\n\t\t\t").concat(generateIconExpressionByStyle(styles.fas), "\n\t\t\ticonLibrary.add(").concat(dynamicIconList, ");\n\t\t\t");
};
var generateIconModule = function generateIconModule(_ref4) {
  var root = _ref4.root,
    relativePath = _ref4.path,
    pattern = _ref4.pattern,
    license = _ref4.license;
  var directoryPath = path.join(root, relativePath);
  var iconRegexPattern = /['"]fa[srbtld]['"], *["'](?:\w-?)+["']/gim;
  if (pattern) {
    iconRegexPattern = new RegExp(pattern, 'gim');
  }
  var iconList = scanIcons(directoryPath, {
    match: iconRegexPattern
  });
  var scripts = generateModuleScript({
    iconList: iconList,
    license: license
  });
  return scripts;
};
module.exports = generateIconModule;